@@ -4,9 +4,15 @@ import base64
 import io
 import os
 import re
+import subprocess  # nosec
+import xml.etree.ElementTree as ET  # nosec
 from contextlib import contextmanager
+from datetime import datetime
 
+import pytz
 import requests
+import svn.local
+import svn.remote
 from git import Repo
 
 from weasel_data_sources.utils import VeryLooseVersion, get_hashes
@@ -107,7 +113,92 @@ class CDNJSFetcher(ReleaseFetcher):
             yield file_object
 
 
-class GitFetcher(ReleaseFetcher):
+class RepositoryFetcher(ReleaseFetcher):
+    """
+    Base fetcher and main logic to retrieve data from Git and SVN repositories
+    """
+
+    def __init__(self, repo_url, local_path, tag_regex=None, tag_substitute=None):
+        self._repo_url = repo_url
+        self._local_path = local_path
+
+        if not os.path.isdir(local_path):
+            self._clone_repository()
+        self._update_and_prepare_repository()
+
+        self._tag_dates = {}
+        self._tag_mapping = {}
+        self._version_pattern = (
+            re.compile(tag_regex, flags=re.IGNORECASE)
+            if tag_regex
+            else re.compile(r"^v?((?:\d+\.\d+).*)$", re.IGNORECASE)
+        )
+
+        # Create a mapping from parsed version strings to tag names
+        for tag in self._get_tags():
+            ver = self._version_pattern.search(tag)
+            if ver is None:
+                print("\tSkipped Tag", tag)
+                continue
+
+            version_str = ver.group(1)
+            if tag_substitute:
+                version_str = re.sub(tag_substitute[0], tag_substitute[1], version_str)
+
+            self._tag_mapping[version_str] = tag
+
+    def _clone_repository(self):
+        raise NotImplementedError
+
+    def _update_and_prepare_repository(self):
+        raise NotImplementedError
+
+    def _get_tags(self):
+        raise NotImplementedError
+
+    def _checkout_release(self, version_str) -> str:
+        raise NotImplementedError
+
+    def get_release_metadata(self) -> list:
+        return self._sort_release_metadata(
+            [
+                {
+                    "version_str": ver,
+                    "publication_date": self._tag_dates[tag].astimezone(pytz.UTC),
+                }
+                for ver, tag in self._tag_mapping.items()
+            ]
+        )
+
+    def get_file_list(self, version_str) -> dict:
+        release_path = self._checkout_release(version_str)
+
+        file_list = {}
+        for root, dirs, files in os.walk(release_path, topdown=True):
+            # Inplace modification will skip excluded directories when iterating over files
+            dirs[:] = [d for d in dirs if d not in {".git"}]
+
+            for file_name in files:
+                full_file_path = os.path.join(root, file_name)
+
+                if os.path.islink(full_file_path):
+                    continue
+
+                repo_file_name = os.path.relpath(full_file_path, release_path)
+                file_list[repo_file_name] = {"hashes": get_hashes(full_file_path)}
+
+        return file_list
+
+    @contextmanager
+    def retrieve_file(self, version_str, filename):
+        release_path = self._checkout_release(version_str)
+        source_path = os.path.join(release_path, filename)
+
+        with open(source_path, "rb") as source_fp:
+            yield source_fp
+
+
+class GitFetcher(RepositoryFetcher):
     """
     Fetcher to retrieve data from arbitrary Git repositories
     """
@@ -121,11 +212,15 @@ class GitFetcher(ReleaseFetcher):
         :param tag_substitute: Tuple of a regex matching parts of the captured version string\
          and a string to replace it with
         """
-        if not os.path.isdir(local_path):
-            Repo.clone_from(repo_url, local_path)
+        self._repo = None
+        self._current_checkout = None
+        super().__init__(repo_url, local_path, tag_regex, tag_substitute)
 
-        self._local_path = local_path
-        self._repo = Repo(local_path)
+    def _clone_repository(self):
+        Repo.clone_from(self._repo_url, self._local_path)
+
+    def _update_and_prepare_repository(self):
+        self._repo = Repo(self._local_path)
 
         # Checkout an arbitrary branch to ensure the following fetch will work
         branch = self._repo.branches[0]
@@ -133,67 +228,68 @@ class GitFetcher(ReleaseFetcher):
         self._current_checkout = branch
 
         self._repo.remotes.origin.pull()
-        self._version_pattern = (
-            re.compile(tag_regex, flags=re.IGNORECASE)
-            if tag_regex
-            else re.compile(r"^v?((?:\d+\.\d+).*)$", re.IGNORECASE)
-        )
 
-        self._tag_mapping = {}
+    def _get_tags(self):
+        tags = set()
         for tag in self._repo.tags:
-            ver = self._version_pattern.search(tag.name)
-            if ver is None:
-                print("\tSkipped Tag", tag.name)
-                continue
+            tags.add(tag.name)
+            self._tag_dates[tag.name] = tag.commit.committed_datetime
 
-            version_str = ver.group(1)
-            if tag_substitute:
-                version_str = re.sub(tag_substitute[0], tag_substitute[1], version_str)
+        return tags
 
-            self._tag_mapping[version_str] = tag
-
-    def _ensure_release(self, version_str):
+    def _checkout_release(self, version_str) -> str:
         tag = self._tag_mapping[version_str]
-        if tag.name != self._current_checkout:
+        if tag != self._current_checkout:
             self._repo.git.checkout(tag)
-            self._current_checkout = tag.name
+            self._current_checkout = tag
 
-    def get_release_metadata(self) -> list:
-        return self._sort_release_metadata(
-            [
-                {
-                    "version_str": ver,
-                    "publication_date": tag.commit.committed_datetime.replace(
-                        tzinfo=None
-                    ),
-                }
-                for ver, tag in self._tag_mapping.items()
-            ]
+        return self._local_path
+
+
+class SVNFetcher(RepositoryFetcher):
+    """
+    Fetcher to retrieve data from arbitrary SVN repositories
+    """
+
+    def __init__(self, repo_url, local_path, tag_regex=None, tag_substitute=None):
+        """
+        :param repo_url: Path to the SVN repository,\
+         i.e. `https://core.svn.wordpress.org/`
+        :param local_path: Path to a local copy of the repo. Will be created if it does not exist.
+        :param tag_regex: Regex capturing versions from tags. Default: r"`^v?((?:\\d+\\.\\d+).*)$`"
+        :param tag_substitute: Tuple of a regex matching parts of the captured version string\
+         and a string to replace it with
+        """
+        self._repo = None
+        super().__init__(repo_url, local_path, tag_regex, tag_substitute)
+
+    def _clone_repository(self):
+        print("Clone repository")
+        repo = svn.remote.RemoteClient(self._repo_url)
+        repo.checkout(self._local_path)
+
+    def _update_and_prepare_repository(self):
+        print("Update repository")
+        self._repo = svn.local.LocalClient(self._local_path)
+        self._repo.update()
+
+    def _get_tags(self):
+        print("Get Tags")
+        svn_output = subprocess.getoutput(
+            "svn ls --xml {}".format(os.path.join(self._local_path, "tags"))
         )
+        tree = ET.fromstring(svn_output)  # nosec
+        tags = []
 
-    def get_file_list(self, version_str) -> dict:
-        self._ensure_release(version_str)
+        for directory in tree.find("list"):
+            tag = directory.find("name").text
+            timestamp = directory.find("./commit/date").text
+            self._tag_dates[tag] = datetime.strptime(
+                timestamp, "%Y-%m-%dT%H:%M:%S.%fZ"
+            ).replace(tzinfo=pytz.UTC)
+            tags.append(tag)
 
-        file_list = {}
-        for root, dirs, files in os.walk(self._local_path, topdown=True):
-            # Inplace modification will skip excluded directories when iterating over files
-            dirs[:] = [d for d in dirs if d not in {".git"}]
+        return tags
 
-            for file_name in files:
-                full_file_path = os.path.join(root, file_name)
-
-                if os.path.islink(full_file_path):
-                    continue
-
-                repo_file_name = os.path.relpath(full_file_path, self._local_path)
-                file_list[repo_file_name] = {"hashes": get_hashes(full_file_path)}
-
-        return file_list
-
-    @contextmanager
-    def retrieve_file(self, version_str, filename):
-        self._ensure_release(version_str)
-        source_path = os.path.join(self._local_path, filename)
-
-        with open(source_path, "rb") as source_fp:
-            yield source_fp
+    def _checkout_release(self, version_str):
+        return os.path.join(self._local_path, "tags", self._tag_mapping[version_str])
